@@ -1,9 +1,49 @@
-`driftmap` <-
-function(long, lat, var, interpol=TRUE, nuage=FALSE,
-listvar=NULL, listnomvar=NULL, carte=NULL, xlab="", col=c("blue","red","blue","pink"),
-pch=c(16,16,16,16), lty=c(1,2),cex=0.7,label=NULL,
-cex.lab=1,axes=FALSE)
+`driftmap` <- function(sp.obj, name.var, interpol=TRUE, nuage=TRUE, lty=1:2, cex=0.7,
+names.attr=names(sp.obj), carte=NULL, identify=FALSE, cex.lab=0.8, pch=rep(16,3),
+col=c("lightblue3", "black", "red"), xlab="", axes=FALSE)
 {
+# Verification of the Spatial Object sp.obj
+class.obj<-class(sp.obj)[1]
+
+if(substr(class.obj,1,7)!="Spatial") stop("sp.obj may be a Spatial object")
+if(substr(class.obj,nchar(class.obj)-8,nchar(class.obj))!="DataFrame") stop("sp.obj should contain a data.frame")
+if(!is.numeric(name.var) & is.na(match(as.character(name.var),names(sp.obj)))) stop("name.var is not included in the data.frame of sp.obj")
+if(length(names.attr)!=length(names(sp.obj))) stop("names.attr should be a vector of character with a length equal to the number of variable")
+
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all=TRUE))==2)
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
+
+# we propose to refind the same arguments used in first version of GeoXp
+long<-coordinates(sp.obj)[,1]
+lat<-coordinates(sp.obj)[,2]
+
+var<-sp.obj@data[,name.var]
+
+# verify the type of the main variable
+if(!(is.integer(var) || is.double(var))) stop("the variable name.var should be a numeric variable")
+
+listvar<-sp.obj@data
+listnomvar<-names.attr
+
+# Code which was necessary in the previous version
+
+ if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
+
+ # for identifyng the selected sites
+ifelse(identify, label<-row.names(listvar),label<-"")
+
 # initialisation
   labvar<-xlab
   nocart<-FALSE
@@ -11,7 +51,7 @@ cex.lab=1,axes=FALSE)
   z<-NULL
   legmap<-NULL
   legends<-list(FALSE,FALSE,"","")
-  
+
   ifelse(interpol, ligne<-"l", ligne<-"p")
   obs<-vector(mode = "logical", length = length(long))
   obs[1:length(obs)]<-TRUE
@@ -24,29 +64,27 @@ cex.lab=1,axes=FALSE)
   method <- ""
   listgraph <- c("Histogram","Barplot","Scatterplot")
   labmod <- ""
-  col2 <- col[3]
+  col2 <- "blue"
   col3 <- col[1]
   pch2 <- pch[3]
-  
+
 
 # initialisation des paramtres modifiables
   theta=0
   nbrow=10
   nbcol=10
-  
-# Ouverture des fentres graphiques  
-graphics.off()
-dev.new()
-dev.new()
 
-  fin <- tclVar(FALSE)
+# Windows device
+if(!(2%in%dev.list())) dev.new()
+if(!(3%in%dev.list())) dev.new()
+
 
 
 ####################################################
 # Changement d'angle
 ####################################################
 
-choixangle <- function() 
+choixangle <- function()
 {
 tt1<-tktoplevel()
 Name <- tclVar("0")
@@ -55,10 +93,10 @@ tkgrid(tklabel(tt1,text="Please enter a angle in degree"),entry.Name)
 
 
 OnOK <- function()
-{ 
+{
 	theta <- tclvalue(Name)
 	tkdestroy(tt1)
-       
+
    if (is.na(as.numeric(theta))||(as.numeric(theta)<0))
      {
       tkmessageBox(message="Sorry, but you have to choose a positive number",icon="warning",type="ok");
@@ -85,23 +123,30 @@ tkfocus(tt1)
 # slection d'un point
 ####################################################
 
-pointfunc<-function() 
+pointfunc<-function()
  {
     quit <- FALSE
 
+    dev.set(2)
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
     while(!quit)
      {
       #slection des points
 
        dev.set(2)
        loc<-locator(1)
-       
+
          if(is.null(loc))
            {
             quit<-TRUE
-            next
+           carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
+           symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
+           cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
+             next
            }
-      
+
        obs<<-selectmap(var1=long,var2=lat,obs=obs,Xpoly=loc[1], Ypoly=loc[2], method="point")
 
        if (length(which(obs==TRUE))==0)
@@ -111,17 +156,20 @@ pointfunc<-function()
        {
       # graphiques
        graphique.drift()
-    
+
       carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
       symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
-      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)]) 
-  
+      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
+
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
             obs=obs, num=4, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2))
         }
-        
+
        }
      }
   }
@@ -131,17 +179,21 @@ pointfunc<-function()
 # slection d'un polygone
 ####################################################
 
-polyfunc<-function() 
+polyfunc<-function()
 {
     polyX <- NULL
     polyY <- NULL
     quit <- FALSE
 
+    dev.set(2)
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
     while(!quit)
     {
         dev.set(2)
         loc<-locator(1)
-        if(is.null(loc)) 
+        if(is.null(loc))
         {
             quit<-TRUE
             next
@@ -150,7 +202,7 @@ polyfunc<-function()
         polyX <- c(polyX, loc[1])
         polyY <- c(polyY, loc[2])
         lines(polyX,polyY)
-    }
+   }
 
     polyX <- c(polyX, polyX[1])
     polyY <- c(polyY, polyY[1])
@@ -162,7 +214,7 @@ polyfunc<-function()
       lines(polyX,polyY)
 
       obs <<- selectmap(var1=long, var2=lat, obs=obs, Xpoly=polyX, Ypoly=polyY, method="poly")
-      
+
       if (length(which(obs==TRUE))==0)
       {SGfunc()
       tkmessageBox(message="You have to select at least 1 site to calculate mean and median",icon="warning",type="ok")
@@ -171,17 +223,17 @@ polyfunc<-function()
       {
       # graphiques
        graphique.drift()
-    
+
       carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
       symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
-      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)]) 
-  
+      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
+
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
             obs=obs, num=4, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2))
         }
-        
+
       }
      }
 }
@@ -190,19 +242,19 @@ polyfunc<-function()
 # contour des units spatiales
 ####################################################
 cartfunc <- function()
-{ 
+{
  if (length(carte) != 0)
-   {  
+   {
     ifelse(!nocart,nocart<<-TRUE,nocart<<-FALSE)
     carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
     symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
-    cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)]) 
-      
+    cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
+
     graphique.drift()
    }
    else
    {
-    tkmessageBox(message="Spatial contours have not been given",icon="warning",type="ok")    
+    tkmessageBox(message="Spatial contours have not been given",icon="warning",type="ok")
    }
 }
 
@@ -213,15 +265,15 @@ cartfunc <- function()
 fbubble<-function()
 {
   res2<-choix.bubble(buble,listvar,listnomvar,legends)
-  
+
   buble <<- res2$buble
   legends <<- res2$legends
   z <<- res2$z
   legmap <<- res2$legmap
-  
+
   carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
   symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
-  cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])  
+  cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
 
 }
 
@@ -230,55 +282,54 @@ fbubble<-function()
 ####################################################
 
 graphfunc <- function()
-{ 
+{
    if ((length(listvar) != 0) && (length(listnomvar) != 0))
     {
-        dev.off(4)
         choix <<- selectgraph(listnomvar,listgraph)
         varChoice1 <<- choix$varChoice1
         varChoice2 <<- choix$varChoice2
         graphChoice <<- choix$graphChoice
-           
+
         if ((graphChoice != "") && (varChoice1 != ""))
         {
-          if (((graphChoice == "Histogram")&&(!is.numeric(listvar[,which(listnomvar == varChoice1)])))||((graphChoice == "Scatterplot")&&((!is.numeric(listvar[,which(listnomvar == varChoice1)]))||(!is.numeric(listvar[,which(listnomvar == varChoice2)]))))) 
+          if (((graphChoice == "Histogram")&&(!is.numeric(listvar[,which(listnomvar == varChoice1)])))||((graphChoice == "Scatterplot")&&((!is.numeric(listvar[,which(listnomvar == varChoice1)]))||(!is.numeric(listvar[,which(listnomvar == varChoice2)])))))
            {
             tkmessageBox(message="Variables choosed are not in a good format",icon="warning",type="ok")
            }
           else
            {
             res1<-choix.couleur(graphChoice,listvar,listnomvar,varChoice1,legends,col,pch)
-            
+
             method <<- res1$method
             col2 <<- res1$col2
             col3 <<- res1$col3
             pch2 <<- res1$pch2
             legends <<- res1$legends
             labmod <<- res1$labmod
-         
+
          if((length(pch2)==1)) pch2<<-pch[3]
-                     
-            dev.new()
+
+            if(!(4%in%dev.list())) dev.new()
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-            obs=obs, num=4, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2))    
-            
+            obs=obs, num=4, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2))
+
       carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
       symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
-      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])  
+      cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
            }
-       }   
+       }
    }
    else
    {
     tkmessageBox(message="Variables (listvar) and their names (listnomvar) must have been given",icon="warning",type="ok")
-   }  
+   }
 }
 
 ####################################################
 # Choisir numbre of grids
 ####################################################
 
-choixgrid <- function() 
+choixgrid <- function()
 {
 tt1<-tktoplevel()
 Name1 <- tclVar("10")
@@ -291,11 +342,11 @@ tkgrid(tklabel(tt1,text="Enter number of columns"),entry.Name2)
 
 
 OnOK <- function()
-{ 
+{
 	nbrow <<- tclvalue(Name1)
 	nbcol <<- tclvalue(Name2)
 	tkdestroy(tt1)
-       
+
    if (is.na(as.numeric(nbrow ))||(as.numeric(nbrow)<2)||(as.numeric(nbcol)<2))
      {
       tkmessageBox(message="Sorry, but you have to choose a number upper than 2",icon="warning",type="ok");
@@ -329,13 +380,13 @@ coords <- cbind(long[obs],lat[obs])
 nvlecoord <- rotation(coords,theta)
 nvlong <- nvlecoord[,1]
 nvlat <- nvlecoord[,2]
-var<-var[obs]
+var.s<-var[obs]
 
-x.lim=c(min(coords[,1],na.rm=TRUE),max(coords[,1],na.rm=TRUE)) 
+x.lim=c(min(coords[,1],na.rm=TRUE),max(coords[,1],na.rm=TRUE))
 y.lim=c(min(coords[,2],na.rm=TRUE),max(coords[,2],na.rm=TRUE))
- 
+
 asp=1/cos((mean(y.lim) * pi)/180)
-if(asp>1.40||asp<0.6) asp=1 
+if(asp>1.40||asp<0.6) asp=1
 
 # cration de la fentre graphique
 # x11(width=7, height=7);
@@ -370,15 +421,15 @@ millig <- NULL
    yind <- c(yind,min(nvlat)+h2*j)
    millig <- c(millig,(yind[j]+yind[j+1])/2)
   }
-  
+
 yind <- c(yind,max(nvlat))
 millig <- c(millig,(yind[nbrow]+yind[nbrow+1])/2)
 
 ###########################################
-# calcul de la moyenne et de la mdiane 
+# calcul de la moyenne et de la mdiane
 ###########################################
 
-# pour chaque colonne 
+# pour chaque colonne
 medcol <- NULL
 moycol <- NULL
 colvide <- rep(FALSE,nbcol)
@@ -388,7 +439,7 @@ ifelse(limmax[nbcol]>0,limmax[nbcol] <- 2*limmax[nbcol],limmax[nbcol] <- limmax[
 
 for (i in 1:nbcol)
 {
- ens <- var[which((nvlong >= xind[i]) & (nvlong < limmax[i]))]
+ ens <- var.s[which((nvlong >= xind[i]) & (nvlong < limmax[i]))]
 
     # si la colonne est vide
     if (length(ens) != 0)
@@ -414,7 +465,7 @@ ifelse(limmax[nbrow]>0,limmax[nbrow]<-2*limmax[nbrow],limmax[nbrow]<-limmax[nbro
 
 for (i in 1:nbrow)
 {
-  ens <- var[which((nvlat >= yind[i]) & (nvlat < limmax[i]))]  
+  ens <- var.s[which((nvlat >= yind[i]) & (nvlat < limmax[i]))]
     # si la ligne est vide
     if (length(ens) != 0)
     {
@@ -430,7 +481,7 @@ for (i in 1:nbrow)
 }
 
 ###########################################
-# dessin de la carte 
+# dessin de la carte
 ###########################################
 
 par(pty="s",mar=c(2,2,1,1))
@@ -449,15 +500,17 @@ if (nocart)
 #    limy <- c(min(nvcarte[,2]), max(nvcarte[,2]));
 
     plot(range(nvlong),range(nvlat),"n", asp=asp)
-    
+
     segments(abs1,ord1,abs2,ord2)
-    points(nvlong,nvlat,col=col[3],pch=pch[3], cex=cex)
+    points(nvlong,nvlat,col=col2,pch=pch[1], cex=cex)
 }
 else
 {
-    plot(nvlong,nvlat,col=col[3],pch=pch[3],asp=asp,cex=cex,xlim=range(nvlong),ylim=range(nvlat))
+    plot(nvlong,nvlat,col=col2,pch=pch[1],asp=asp,cex=cex,xlim=range(nvlong),ylim=range(nvlat))
 }
 
+     opt.usr<-par()$usr
+
 # dessin de la grille
 # colonnes
 for (i in 1:(nbcol+1))
@@ -473,50 +526,58 @@ for (i in 1:(nbrow+1))
 
 
 ###########################################
-# dessin des graphiques 
+# dessin des graphiques
 ###########################################
 
 # lignes
-par(mar=c(2,0,1,1))
+op<-par(mar=c(2,0,1,1))
 
 if (!nuage)
 {
- plot(var,nvlat,"n",xlim=c(min(moylig[!ligvide],medlig[!ligvide]),max(moylig[!ligvide],medlig[!ligvide])),
-  ylim=range(nvlat),yaxt="n")  
+# plot(var.s,nvlat,"n",xlim=c(min(moylig[!ligvide],medlig[!ligvide]),max(moylig[!ligvide],medlig[!ligvide])),
+#  ylim=range(nvlat),yaxt="n")
+   plot(var.s,nvlat,"n",yaxt="n")
+   op<-par(usr=c(par()$usr[1:2],opt.usr[3:4]))
 }
 else
 {
- plot(var,nvlat,col=col[4],pch=pch[4],cex=cex,xlim=range(var),ylim=range(nvlat),yaxt="n")
+# plot(var.s,nvlat,col=col[1],pch=pch[1],cex=cex,xlim=range(var.s),ylim=range(nvlat),yaxt="n")
+  plot(var.s,nvlat,type="n",col=col[1],pch=pch[1],cex=cex,yaxt="n")
+  op<-par(usr=c(par()$usr[1:2],opt.usr[3:4]))
+  points(var.s,nvlat,col=col[1],pch=pch[1],cex=cex,yaxt="n")
 }
 
 
-    points(moylig[!ligvide],millig[!ligvide],col=col[1],pch=pch[1])
-    points(moylig[!ligvide],millig[!ligvide],col=col[1],pch=pch[1],type=ligne,lty=lty[1])
-    
-    points(medlig[!ligvide],millig[!ligvide],col=col[2],pch=pch[2])
-    points(medlig[!ligvide],millig[!ligvide],col=col[2],pch=pch[2],type=ligne,lty=lty[2])
+    points(moylig[!ligvide],millig[!ligvide],col=col[2],pch=pch[2])
+    points(moylig[!ligvide],millig[!ligvide],col=col[2],pch=pch[2],type=ligne,lty=lty[1])
+
+    points(medlig[!ligvide],millig[!ligvide],col=col[3],pch=pch[3])
+    points(medlig[!ligvide],millig[!ligvide],col=col[3],pch=pch[3],type=ligne,lty=lty[2])
 
 # colonnes
 par(mar=c(0,2,1,1))
 
 if (!nuage)
 {
- plot(nvlong,var,"n",ylim=c(min(moycol[!colvide],medcol[!colvide]),max(moycol[!colvide],medlig[!colvide])),
-  xlim=range(nvlong),xaxt="n",yaxt="n")  
- axis(side=4) 
+ plot(nvlong,var.s,"n",ylim=c(min(moycol[!colvide],medcol[!colvide]),max(moycol[!colvide],medlig[!colvide])),
+  xlim=range(nvlong),xaxt="n",yaxt="n")
+ axis(side=4)
+ op<-par(usr=c(opt.usr[1:2],par()$usr[3:4]))
 }
 else
 {
- plot(nvlong,var,col=col[4],pch=pch[4],cex=cex,ylim=range(var),xlim=range(nvlong),xaxt="n",yaxt="n")
+ plot(nvlong,var.s,type="n",col=col[1],pch=pch[1],cex=cex,ylim=range(var.s),xlim=range(nvlong),xaxt="n",yaxt="n")
  axis(side=4)
+ op<-par(usr=c(opt.usr[1:2],par()$usr[3:4]))
+ points(nvlong,var.s,col=col[1],pch=pch[1],cex=cex,ylim=range(var.s),xlim=range(nvlong),xaxt="n",yaxt="n")
 }
 
- points(milcol[!colvide],moycol[!colvide],col=col[1],pch=pch[1])
- points(milcol[!colvide],moycol[!colvide],col=col[1],pch=pch[1],type=ligne,lty=lty[1])
- 
- points(milcol[!colvide],medcol[!colvide],col=col[2],pch=pch[2])
- points(milcol[!colvide],medcol[!colvide],col=col[2],pch=pch[2],type=ligne,lty=lty[2])
- 
+ points(milcol[!colvide],moycol[!colvide],col=col[2],pch=pch[2])
+ points(milcol[!colvide],moycol[!colvide],col=col[2],pch=pch[2],type=ligne,lty=lty[1])
+
+ points(milcol[!colvide],medcol[!colvide],col=col[3],pch=pch[3])
+ points(milcol[!colvide],medcol[!colvide],col=col[3],pch=pch[3],type=ligne,lty=lty[2])
+
 ###########################################
 # prcision des indications
 ###########################################
@@ -537,7 +598,7 @@ x<-nvlecoord.arrox[,1]
 y<-nvlecoord.arrox[,2]
 
 # drawing polygons
-for (i in 1:15) 
+for (i in 1:15)
 {
 x1 <- c(x[i],x[i+1],loc[1])
 y1 <- c(y[i],y[i+1],loc[2])
@@ -559,7 +620,7 @@ text(new.let,b,cex=cex.lab)
 
 # lgende
 
-legend(-1.75,1.75, c("Mean","Median"), col=col, pch=pch,lty=lty, cex=cex.lab)
+legend(-1.75,1.75, c("Mean","Median"), col=col[c(2,3)], pch=pch[c(2,3)],lty=lty, cex=cex.lab)
 
 # nom de la variable tudie
 if (labvar != "")
@@ -576,37 +637,51 @@ if (labvar != "")
 # quitter l'application
 ####################################################
 
-    quitfunc <- function() {
-        tclvalue(fin) <<- TRUE
-      #  graphics.off()
-        tkdestroy(tt)
+quitfunc<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    dev.set(3)
+    layout(1)
+    par(mar=c(5.1,4.1,4.1,2.1))
+   # assign("obs", row.names(sp.obj)[obs], envir = .GlobalEnv)
+}
 
-   #    tkdestroy(tt1)
-   #tkwait.variable(fin)
-    }
+quitfunc2<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    dev.set(3)
+    layout(1)
+    par(mar=c(5.1,4.1,4.1,2.1))
+    print("Results have been saved in last.select object")
+    assign("last.select", which(obs), envir = .GlobalEnv)
+}
 
 ####################################################
 # rafraichissement des graphiques
 ####################################################
 
-SGfunc<-function() 
+SGfunc<-function()
 {
  obs<<-vector(mode = "logical", length = length(long))
  obs[1:length(obs)]<<-TRUE
-  
+
  # graphiques
  graphique.drift()
- 
+
  carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,label=label,
  symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes, labmod=labmod,
- cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])  
+ cex.lab=cex.lab,method=method,classe=listvar[,which(listnomvar == varChoice1)])
 
  if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
   {
    graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
    obs=obs, num=4, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2))
   }
-          
+
 }
 
 ####################################################
@@ -615,102 +690,80 @@ SGfunc<-function()
 
 graphique.drift()
 carte(long=long, lat=lat, obs=obs,label=label,symbol=pch[3],carte=carte,nocart=nocart,
-cex.lab=cex.lab,method="") 
+cex.lab=cex.lab,method="",axis=axes)
 
 ####################################################
 # cration de la boite de dialogue
 ####################################################
+
 if(interactive())
 {
+fontheading<-tkfont.create(family="times",size=14,weight="bold")
+
 tt <- tktoplevel()
+tkwm.title(tt, "driftmap")
 
-labelText1 <- tclVar("Work on the map")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
+frame1a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1a, text = "Interactive selection", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame1a, text = "Work on the map", font = "Times 12",
+foreground = "darkred", background = "white"))
 
-point.but <- tkbutton(tt, text="  Point  ", command=pointfunc);
-poly.but <- tkbutton(tt, text=" Polygon ", command=polyfunc);
-tkgrid(point.but, poly.but)
-tkgrid(tklabel(tt,text="    "))
+point.but <- tkbutton(frame1a, text="Selection by point", command=pointfunc);
+poly.but <- tkbutton(frame1a, text="Selection by polygon ", command=polyfunc);
+tkpack(point.but, poly.but, side = "left", expand = "TRUE",
+        fill = "x")
+
+tkpack(frame1a, expand = "TRUE", fill = "x")
+
+frame1c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1c, text = "Work on the graphic", font = "Times 12",
+foreground = "darkred", background = "white"))
+vari12.but <- tkbutton(frame1c, text="Change the number of cells", command=choixgrid);
+vari1.but <- tkbutton(frame1c, text="Change the rotation angle", command=choixangle);
+tkpack(vari12.but,vari1.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1c, expand = "TRUE", fill = "x")
 
 
-label1 <- tclVar("To stop selection, leave the cursor on the active graph, click on the right button of the mouse and stop")
-label11 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(label1))
-tkconfigure(label11, textvariable=label1)
-tkgrid(label11,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+frame1b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nettoy.but <- tkbutton(frame1b, text="     Reset selection     " , command=SGfunc);
+tkpack(nettoy.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1b, expand = "TRUE", fill = "x")
 
 
-labelText12 <- tclVar("Change the number of cells")
-label12 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText12))
-tkconfigure(label12, textvariable=labelText12)
-tkgrid(label12,columnspan=2)
+frame2 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2, text = "Options", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame2, text = "Spatial contours", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Bubbles", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Additional graph", font = "Times 11",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2, expand = "TRUE", fill = "x")
+
+frame2b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nocou1.but <- tkbutton(frame2b, text="On/Off", command=cartfunc)
+bubble.but <- tkbutton(frame2b, text="On/Off", command=fbubble)
+autre.but <- tkbutton(frame2b, text="     OK     " , command=graphfunc)
+
+tkpack(nocou1.but,bubble.but,autre.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2b, expand = "TRUE", fill = "x")
 
 
-vari12.but <- tkbutton(tt, text="     OK     ", command=choixgrid);
-tkgrid(vari12.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+frame3 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame3, text = "Exit", font = "Times 14",
+foreground = "blue", background = "white"))
 
-labelText1 <- tclVar("Change the rotation angle")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
+quit.but <- tkbutton(frame3, text="Save results", command=quitfunc2);
+quit.but2 <- tkbutton(frame3, text="Exit without saving", command=quitfunc);
 
+tkpack(quit.but, quit.but2, side = "left", expand = "TRUE",
+        fill = "x")
 
-vari1.but <- tkbutton(tt, text="     OK     ", command=choixangle);
-tkgrid(vari1.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText6 <- tclVar("Draw spatial contours")
-label6 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText6))
-tkconfigure(label6, textvariable=labelText6)
-tkgrid(label6,columnspan=2)
-    
-nocou1.but <- tkbutton(tt, text="  On/Off  ", command=cartfunc);
-tkgrid(nocou1.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText9 <- tclVar("Bubbles")
-label9 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText9))
-tkconfigure(label9, textvariable=labelText9)
-tkgrid(label9,columnspan=2)
-
-bubble.but <- tkbutton(tt, text="  On/Off  ", command=fbubble);
-tkgrid(bubble.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText3 <- tclVar("Restore graph")
-label3 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText3))
-tkconfigure(label3, textvariable=labelText3)
-tkgrid(label3,columnspan=2)
-
-nettoy.but <- tkbutton(tt, text="     OK     " , command=SGfunc);
-tkgrid(nettoy.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText4 <- tclVar("Additional graph")
-label4 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText4))
-tkconfigure(label4, textvariable=labelText4)
-tkgrid(label4,columnspan=2)
-
-autre.but <- tkbutton(tt, text="     OK     " , command=graphfunc);
-tkgrid(autre.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText5 <- tclVar("Exit")
-label5 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText5))
-tkconfigure(label5, textvariable=labelText5)
-tkgrid(label5,columnspan=2)
-
-quit.but <- tkbutton(tt, text="     OK     ", command=quitfunc);
-tkgrid(quit.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-####################################################
-
-tkwait.variable(fin)
+tkpack(frame3, expand = "TRUE", fill = "x")
 }
 
+##############################################
+return(invisible())
+
 }
 
