@@ -1,10 +1,61 @@
 `moranplotmap` <-
-function (long, lat, var, W, flower=FALSE, locmoran=FALSE, listvar=NULL, listnomvar=NULL,carte=NULL,
-criteria=NULL, label="", cex.lab=1, col="blue",pch=16, xlab=expression((X-bar(X))),ylab=expression(W(X-bar(X))),
-axes=FALSE, lablong="", lablat="",names.arg=c("H.-H.","L.-H.","L.-L.","H.-L."))
+function(sp.obj, name.var, listw.obj, flower=FALSE, locmoran=FALSE, names.arg=c("H.-H.","L.-H.","L.-L.","H.-L."),
+names.attr=names(sp.obj), criteria=NULL, carte=NULL, identify=FALSE, cex.lab=0.8, pch=16, col="lightblue3",
+xlab="", ylab="", axes=FALSE, lablong="", lablat="")
 {
+# Verification of the Spatial Object sp.obj
+class.obj<-class(sp.obj)[1]
+
+if(substr(class.obj,1,7)!="Spatial") stop("sp.obj may be a Spatial object")
+if(substr(class.obj,nchar(class.obj)-8,nchar(class.obj))!="DataFrame") stop("sp.obj should contain a data.frame")
+if(!is.numeric(name.var) & is.na(match(as.character(name.var),names(sp.obj)))) stop("name.var is not included in the data.frame of sp.obj")
+if(length(names.attr)!=length(names(sp.obj))) stop("names.attr should be a vector of character with a length equal to the number of variable")
+
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all=TRUE))==2) # new environment
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
+
+# we propose to refind the same arguments used in first version of GeoXp
+long<-coordinates(sp.obj)[,1]
+lat<-coordinates(sp.obj)[,2]
+
+var<-sp.obj@data[,name.var]
+
+# verify the type of the main variable
+if(!(is.integer(var) || is.double(var))) stop("the variable name.var should be a numeric variable")
+
+listvar<-sp.obj@data
+listnomvar<-names.attr
+
+# Code which was necessary in the previous version
+ if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
+
+ # for identifyng the selected sites
+ifelse(identify, label<-row.names(listvar),label<-"")
+
+# We create a spatial weight matrix by using a matrix object
+  n <- nrow(sp.obj)
+  W<-matrix(0,n,n)
+  W.sn<-listw2sn(listw.obj)
+  W[as.matrix(W.sn[,1:2])]<-W.sn[,3]
+
+# Is W normalized ?
+  is.norm<-all(apply(W,1,sum)==rep(1,n))
 
 #initialisation
+if(xlab=="") xlab=name.var
+if(ylab=="") ylab=paste("spatially lagged",name.var)
   obs<-vector(mode = "logical", length = length(long))
   obsq<-rep(0,length(long))
   nointer<-FALSE
@@ -30,12 +81,10 @@ axes=FALSE, lablong="", lablat="",names.arg=c("H.-H.","L.-H.","L.-L.","H.-L."))
   choix <- ""
   listgraph <- c("Histogram","Barplot","Scatterplot")
 
-#Ouverture des fentres graphiques
-graphics.off()
-dev.new()
-dev.new()
+# Windows device
+if(!(2%in%dev.list())) dev.new()
+if(!(3%in%dev.list())) dev.new()
 
-fin <- tclVar(FALSE)
 
 quad <- FALSE
 
@@ -49,15 +98,15 @@ choix.col<-FALSE
 
 graph <- "Moran"
 col2 <- rep(col[1],4)
+col3 <- rep("blue",4)
 pch2 <- rep(pch[1],4)
 
 
-#Normalize w
-#W<-normw(W)
-ifelse(apply(W,1,sum)==rep(1,length(long)),is.norm<-TRUE,is.norm<-FALSE)
+
 # calcul du I de Moran
-var <- var - mean(var)
+#var <- var - mean(var)
 wvar <- W%*%var
+
 stdvar <- var/sd(var)
 uns <- rep(1, length(var))
 result <- nonormmoran(stdvar,uns,W)
@@ -68,10 +117,10 @@ beta.I <- qr.coef(rvar,wvar)
 
 # calcul de la variable obsq (pour les quadrants)
 
-obsq[which((var > 0) & (wvar >= 0))] <- 1
-obsq[which((var <= 0) & (wvar > 0))] <- 2
-obsq[which((var < 0) & (wvar <= 0))] <- 3
-obsq[which((var >= 0) & (wvar < 0))] <- 4
+obsq[which((var > mean(var)) & (wvar >= mean(wvar)))] <- 1
+obsq[which((var <= mean(var)) & (wvar > mean(wvar)))] <- 2
+obsq[which((var < mean(var)) & (wvar <= mean(wvar)))] <- 3
+obsq[which((var >= mean(var)) & (wvar < mean(wvar)))] <- 4
  
   
 ####################################################
@@ -82,7 +131,18 @@ pointfunc<-function()
 {
     quit <- FALSE
     quad <<- FALSE
-
+ 
+    if(maptest) 
+      { dev.set(2)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+      }
+     else
+     { dev.set(3)
+       title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+       title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+      
     while(!quit)
     {
         if(maptest) 
@@ -92,6 +152,10 @@ pointfunc<-function()
             if(is.null(loc)) 
             {
               quit<-TRUE
+              carte(long=long, lat=lat, obs=obs,  carte=carte,nocart=nocart, classe=obsq,
+              couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+              nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
+              label=label, cex.lab=cex.lab, labmod=names.arg)   
               next
             }           
             obs<<-selectmap(var1=long,var2=lat,obs=obs,Xpoly=loc[1], Ypoly=loc[2], method="point") 
@@ -103,6 +167,9 @@ pointfunc<-function()
             if(is.null(loc)) 
             {
               quit<-TRUE
+              graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
+              symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
+              legmap=legmap2, legends=legends2, bin=is.norm)
               next
             }
             obs<<-selectmap(var1=var,var2=wvar,obs=obs,Xpoly=loc[1], Ypoly=loc[2], method="point")
@@ -110,14 +177,26 @@ pointfunc<-function()
         
         #graphiques
         carte(long=long, lat=lat, obs=obs,  carte=carte,nocart=nocart, classe=obsq,
-        couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+        couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
         nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
         label=label, cex.lab=cex.lab, labmod=names.arg)      
 
-        graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+        graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
         symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-        legmap=legmap2, legends=legends2 ) 
-           
+        legmap=legmap2, legends=legends2, bin=is.norm)
+      
+      if(maptest) 
+      { dev.set(2)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+      }
+     else
+     { dev.set(3)
+       title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+       title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+     
+        
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)], 
@@ -132,7 +211,7 @@ pointfunc<-function()
 
 pt1func <- function()
 {
-  ifelse(flower,method <<- "Neighbourplot1", method <<- "")
+  ifelse(flower,method <<- "Neighbourplot1", method <<- "Quadrant")
   graph <<- "Moran" 
   maptest <<- TRUE  
   pointfunc()
@@ -144,7 +223,7 @@ pt1func <- function()
 
 pt2func <- function()
 {    
-  ifelse(flower,method <<- "Neighbourplot1", method <<- "")
+  ifelse(flower,method <<- "Neighbourplot1", method <<- "Quadrant")
   graph <<- "Moran" 
   maptest <<- FALSE  
   pointfunc()
@@ -160,7 +239,18 @@ polyfunc<-function()
    polyY <- NULL
    quit <- FALSE
    quad <<- FALSE
-
+   
+   if(maptest) 
+      { dev.set(2)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+      }
+     else
+     { dev.set(3)
+       title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+       title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+    }
+      
    while(!quit)
     {
         if(maptest)
@@ -205,13 +295,13 @@ polyfunc<-function()
 
     #graphiques
         carte(long=long, lat=lat, obs=obs,  carte=carte,nocart=nocart, classe=obsq,
-        couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+        couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
         nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
         label=label, cex.lab=cex.lab, labmod=names.arg)        
 
-        graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+        graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
         symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-        legmap=legmap2, legends=legends2 )   
+        legmap=legmap2, legends=legends2, bin=is.norm)
         
      if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
@@ -274,12 +364,12 @@ quadfunc <- function()
     obs[which(obsq == num)] <<- !obs[which(obsq == num)]
 
     carte(long=long, lat=lat, obs=obs, classe=obsq,  carte=carte,nocart=nocart, labmod=names.arg,
-    couleurs=col2,symbol=pch2, method="Quadrant",buble=buble,cbuble=z,criteria=criteria,nointer=nointer,legmap=legmap,
+    couleurs=col3,symbol=pch2, method="Quadrant",buble=buble,cbuble=z,criteria=criteria,nointer=nointer,legmap=legmap,
     legends=legends,axis=axes,lablong=lablong, lablat=lablat, label=label, cex.lab=cex.lab) 
 
-    graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+    graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
     symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-    legmap=legmap2, legends=legends2 )   
+    legmap=legmap2, legends=legends2, bin=is.norm)
     
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
      {
@@ -332,26 +422,34 @@ quad4func <- function()
 {
     if(!choix.col)
     {choix.col<<-TRUE
+     method <<- "Quadrant"
      res1<-choix.couleur("Moran",col=col[1],pch=pch[1],legends=legends)     
-     ifelse(length(res1$col2)==4,col2 <<- res1$col2,col2 <<- rep(col[1],4))
+     if(length(res1$col2)==4)
+      {col2 <<- res1$col2
+       col3 <<- col2 }
+      else
+      {col2 <<- rep(col[1],4)
+       col3 <<- rep("blue",4) }
+      
      ifelse(length(res1$pch2)==4,pch2 <<- res1$pch2,pch2 <<- rep(pch[1],4))
      legends <<- res1$legends
      }
     else
     {choix.col<<-FALSE
      col2 <<- rep(col[1],4)
+     col3 <<- rep("blue",4)
      pch2 <<- rep(pch[1],4)
      legends <<- list(legends[[1]],FALSE,legends[[3]],"")
     }
-     
+
 carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=obsq,
-couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
 nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
 label=label, cex.lab=cex.lab,labmod=names.arg)     
 
-graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
 symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-legmap=legmap2, legends=legends2 )   
+legmap=legmap2, legends=legends2, bin=is.norm)
         
   }
 
@@ -363,7 +461,6 @@ graphfunc <- function()
 {
     if ((length(listvar) != 0) && (length(listnomvar) != 0))
     {
-        dev.off(4)
         choix <<- selectgraph(listnomvar,listgraph)
         varChoice1 <<- choix$varChoice1
         varChoice2 <<- choix$varChoice2
@@ -371,7 +468,7 @@ graphfunc <- function()
             
         if ((graphChoice != "") && (varChoice1 != ""))
         {
-            dev.new()
+            if(!(4%in%dev.list())) dev.new()
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
             obs=obs, num=4, graph=graphChoice, couleurs=col[1], symbol=pch[1], labvar=c(varChoice1,varChoice2))
         }
@@ -394,7 +491,7 @@ cartfunc <- function()
    {
     ifelse(!nocart,nocart<<-TRUE,nocart<<-FALSE)
     carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=obsq,
-    couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+    couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
     nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
     label=label, cex.lab=cex.lab,labmod=names.arg)        
    }
@@ -415,18 +512,19 @@ SGfunc<-function()
     obs<<-vector(mode = "logical", length = length(long));
 
 carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=obsq,
-couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
 nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
 label=label, cex.lab=cex.lab,labmod=names.arg)     
 
-graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
 symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-legmap=legmap2, legends=legends2 )   
+legmap=legmap2, legends=legends2, bin=is.norm)
     
-   if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
-        {
-            graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)], obs=obs, num=4, graph=graphChoice, symbol=1, labvar=c(varChoice1,varChoice2));
-        }
+if ((graphChoice != "") && (varChoice1 != ""))
+ {
+ graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
+ obs=obs, num=4, graph=graphChoice, couleurs=col[1], symbol=pch[1], labvar=c(varChoice1,varChoice2))
+}
 
 }
 
@@ -434,10 +532,22 @@ legmap=legmap2, legends=legends2 )
 # quitter l'application
 ####################################################
 
-quitfunc<-function() 
+quitfunc<-function()
 {
-  tclvalue(fin)<<-TRUE
-  tkdestroy(tt)
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+   # assign("obs", row.names(sp.obj)[obs], envir = .GlobalEnv)
+}
+
+quitfunc2<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    print("Results have been saved in last.select object")
+    res<-list(obs=which(obs),MORAN=MORAN)
+    assign("last.select", res, envir = .GlobalEnv)
 }
 
 ####################################################
@@ -450,7 +560,7 @@ fnointer<-function()
  {
  ifelse(!nointer,nointer<<-TRUE,nointer<<-FALSE)
  carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=obsq,
- couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+ couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
  nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
  label=label, cex.lab=cex.lab,labmod=names.arg)       
  }
@@ -476,7 +586,7 @@ fbubble<-function()
   legmap <<- res2$legmap
   
 carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=obsq,
-couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
 nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
 label=label, cex.lab=cex.lab,labmod=names.arg)    
  
@@ -496,9 +606,9 @@ lisa<-function()
   z2 <<- res3$z
   legmap2 <<- res3$legmap
   
-graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2, 
+graphique(var1=var, var2=wvar, obs=obs, num=3, graph=graph, labvar=labvar, couleurs=col2,
 symbol=pch2, locmoran=locmoran,obsq=obsq, cex.lab=cex.lab,buble=buble2, cbuble=z2, 
-legmap=legmap2, legends=legends2 )   
+legmap=legmap2, legends=legends2, bin=is.norm)
  
 }
 
@@ -546,138 +656,124 @@ permutation<-function()
 }
 
 carte(long=long, lat=lat, obs=obs, carte=carte,nocart=nocart, classe=classe,
-couleurs=col2, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
+couleurs=col3, symbol=pch2, W=W, method=method, buble=buble, cbuble=z, criteria=criteria,
 nointer=nointer, legmap=legmap, legends=legends,axis=axes,lablong=lablong, lablat=lablat,
 label=label, cex.lab=cex.lab)     
 
-graphique(var1=var, var2=wvar, obs=obs, num=3, graph="Moran", labvar=labvar, couleurs=col2, symbol=pch2, 
-locmoran=locmoran,obsq=obsq,buble=buble2, cbuble=z2, legmap=legmap2, legends=legends2 )
+graphique(var1=var, var2=wvar, obs=obs, num=3, graph="Moran", labvar=labvar, couleurs=col2, symbol=pch2,
+locmoran=locmoran,obsq=obsq,buble=buble2, cbuble=z2, legmap=legmap2, legends=legends2, bin=is.norm)
         
 
 ####################################################
 # cration de la boite de dialogue
 ####################################################
+
 if(interactive())
 {
+fontheading<-tkfont.create(family="times",size=14,weight="bold")
+
 tt <- tktoplevel()
+tkwm.title(tt, "moranplotmap")
 
-labelText1 <- tclVar("Work on the map")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
+frame1a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1a, text = "Interactive selection", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame1a, text = "Work on the map", font = "Times 12",
+foreground = "darkred", background = "white"))
+point.but <- tkbutton(frame1a, text="Selection by point", command=pt1func);
+poly.but <- tkbutton(frame1a, text="Selection by polygon ", command=poly1func);
+tkpack(point.but, poly.but, side = "left", expand = "TRUE",fill = "x")
+tkpack(frame1a, expand = "TRUE", fill = "x")
 
-point.but <- tkbutton(tt, text="  Points  ", command=pt1func);
-poly.but <- tkbutton(tt, text=" Polygon ", command=poly1func);
-tkgrid(point.but, poly.but)
-tkgrid(tklabel(tt,text="    "))
+frame1c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1c, text = "Work on the graphic", font = "Times 12",
+foreground = "darkred", background = "white"))
+point2.but <- tkbutton(frame1c, text="Selection by point", command=pt2func);
+poly2.but <- tkbutton(frame1c, text="Selection by polygon ", command=poly2func);
+tkpack(point2.but, poly2.but, side = "left", expand = "TRUE",fill = "x")
+tkpack(frame1c, expand = "TRUE", fill = "x")
+
+frame1d <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1d, text = "Select a quadrant", font = "Times 12",
+foreground = "darkred", background = "white"))
+HH.but <- tkbutton(frame1d, text="H.-H.", command=quad1func);
+LH.but <- tkbutton(frame1d, text="H.-L.", command=quad4func);
+LL.but <- tkbutton(frame1d, text="L.-L.", command=quad3func);
+HL.but <- tkbutton(frame1d, text="L.-H.", command=quad2func);
+tkpack(HH.but,LH.but,LL.but,HL.but, side = "left", expand = "TRUE",fill = "x")
+tkpack(frame1d, expand = "TRUE", fill = "x")
+
+frame1b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nettoy.but <- tkbutton(frame1b, text="     Reset selection     " , command=SGfunc);
+tkpack(nettoy.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1b, expand = "TRUE", fill = "x")
 
 
-labelText1 <- tclVar("Work on the moranplot")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
-
-point1.but <- tkbutton(tt, text="  Points  ", command=pt2func);
-poly1.but <- tkbutton(tt, text=" Polygon ", command=poly2func);
-tkgrid(point1.but, poly1.but)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText1 <- tclVar("Choose a quadrant")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
-
-point2.but <- tkbutton(tt, text="4th Quadrant", command=quad2func);
-poly2.but <- tkbutton(tt, text="1st Quadrant", command=quad1func);
-tkgrid(point2.but, poly2.but)
-point3.but <- tkbutton(tt, text="3rd Quadrant", command=quad3func);
-poly3.but <- tkbutton(tt, text="2nd Quadrant", command=quad4func);
-tkgrid(point3.but, poly3.but)
-
-tkgrid(tklabel(tt,text="    "))
-
-
-
-label1 <- tclVar("To stop selection, leave the cursor on the active graph, click on the right button of the mouse and stop")
-label11 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(label1))
-tkconfigure(label11, textvariable=label1)
-tkgrid(label11,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
+frame11a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame11a, text = "Moran test", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(frame11a, expand = "TRUE", fill = "x")
 
+frame11b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
 msg <- paste("Moran index ",ifelse(is.norm,"(W normalized)","(W not normalized)"),": ", MORAN, " - ","p-value (Gaussian Test) : ", ifelse(round(1-prob.I,4)<0.0001,"<0.0001",round(1-prob.I,4)))
-tkgrid(tklabel(tt,text=msg),columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+tkgrid(tklabel(frame11b,text=msg),columnspan=2)
+tkpack(frame11b, expand = "TRUE", fill = "x")
 
-
-labelText10 <- tclVar("Permutation Test")
-label10 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText10))
-tkconfigure(label10, textvariable=labelText10)
-tkgrid(label10,columnspan=2)
-
-noint10.but <- tkbutton(tt, text="     OK     ", command=permutation);
-tkgrid(noint10.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText7 <- tclVar("Preselected sites  /  Draw Spatial contours ")
-label7 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText7))
-tkconfigure(label7, textvariable=labelText7)
-tkgrid(label7,columnspan=2)
-
-noint1.but <- tkbutton(tt, text="  On/Off  ", command=fnointer);
-nocou1.but <- tkbutton(tt, text="  On/Off  ", command=cartfunc);
-tkgrid(noint1.but , nocou1.but)
-tkgrid(tklabel(tt,text="    "))
+frame11c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+noint10.but <- tkbutton(frame11c, text="Permutation Test", command=permutation);
+tkpack(noint10.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame11c, expand = "TRUE", fill = "x")
 
 
 
-labelText9 <- tclVar("Print different colors by quadrant / Bubbles")
-label9 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText9))
-tkconfigure(label9, textvariable=labelText9)
-tkgrid(label9,columnspan=2)
+frame2 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2, text = "Options", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame2, text = "Spatial contours", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Preselected sites", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Bubbles", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Additional graph", font = "Times 10",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2, expand = "TRUE", fill = "x")
 
-coul.but <- tkbutton(tt, text="  On/Off  ", command=colfunc);
-bubble.but <- tkbutton(tt, text="  On/Off  ", command=fbubble);
-tkgrid(coul.but,bubble.but)
-tkgrid(tklabel(tt,text="    "))
-
-labelText3 <- tclVar("Restore graph")
-label3 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText3))
-tkconfigure(label3, textvariable=labelText3)
-tkgrid(label3,columnspan=2)
-
-nettoy.but <- tkbutton(tt, text="     OK     " , command=SGfunc);
-tkgrid(nettoy.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+frame2b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nocou1.but <- tkbutton(frame2b, text="On/Off", command=cartfunc)
+noint1.but <- tkbutton(frame2b, text="On/Off", command=fnointer)
+bubble.but <- tkbutton(frame2b, text="On/Off", command=fbubble)
+autre.but <- tkbutton(frame2b, text="     OK     " , command=graphfunc)
+tkpack(nocou1.but,noint1.but,bubble.but,autre.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2b, expand = "TRUE", fill = "x")
 
 
-labelText4 <- tclVar("Bubbles of LISA / Additional graph")
-label4 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText4))
-tkconfigure(label4, textvariable=labelText4)
-tkgrid(label4,columnspan=2)
 
-lisa.but <- tkbutton(tt, text="  On/Off  ", command=lisa);
-autre.but <- tkbutton(tt, text="     OK     " , command=graphfunc);
-tkgrid(lisa.but,autre.but)
-tkgrid(tklabel(tt,text="    "))
+frame2e <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2e, text = "Print different colors by quadrant  ", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2e, text = "Bubles of LISA", font = "Times 10",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2e, expand = "TRUE", fill = "x")
 
+frame2f <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+colquad.but <- tkbutton(frame2f, text="On/Off", command=colfunc)
+lisa.but <- tkbutton(frame2f, text="     OK     " , command=lisa)
+tkpack(colquad.but,lisa.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2f, expand = "TRUE", fill = "x")
 
-labelText5 <- tclVar(" Exit ")
-label5 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText5))
-tkconfigure(label5, textvariable=labelText5)
-tkgrid(label5,columnspan=2)
+frame3 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame3, text = "Exit", font = "Times 14",
+foreground = "blue", background = "white"))
 
-quit.but <- tkbutton(tt, text="     OK     ", command=quitfunc);
-tkgrid(quit.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+quit.but <- tkbutton(frame3, text="Save results", command=quitfunc2);
+quit.but2 <- tkbutton(frame3, text="Exit without saving", command=quitfunc)
 
-tkwait.variable(fin)
+tkpack(quit.but, quit.but2, side = "left", expand = "TRUE",
+        fill = "x")
+
+tkpack(frame3, expand = "TRUE", fill = "x")
 }
+
+
 ####################################################
+return(invisible())
 
-
-return(list(obs=obs,MORAN=MORAN))
-  }
+}
 
