@@ -1,8 +1,50 @@
- polyboxplotmap<-
-function (long, lat, var1, var2, varwidth=FALSE,listvar = NULL, listnomvar = NULL,
-carte = NULL, criteria = NULL, label = "", cex.lab=1, names.arg = "", xlab = "",
-ylab= "count", pch = 16, col = "grey", axes = FALSE, lablong = "", lablat = "")
+ polyboxplotmap<-function(sp.obj, names.var, varwidth=FALSE, names.arg = "",
+names.attr=names(sp.obj), criteria=NULL, carte=NULL, identify=FALSE, cex.lab=0.8,
+pch=16, col="lightblue3",xlab="", ylab="count", axes=FALSE, lablong="", lablat="")
 {
+# Verification of the Spatial Object sp.obj
+class.obj<-class(sp.obj)[1]
+
+if(substr(class.obj,1,7)!="Spatial") stop("sp.obj may be a Spatial object")
+if(substr(class.obj,nchar(class.obj)-8,nchar(class.obj))!="DataFrame") stop("sp.obj should contain a data.frame")
+if(!is.numeric(names.var) & length(match(names.var,names(sp.obj)))!=length(names.var) ) stop("At least one component of names.var is not included in the data.frame of sp.obj")
+if(length(names.attr)!=length(names(sp.obj))) stop("names.attr should be a vector of character with a length equal to the number of variable")
+
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all=TRUE))==2)  # new environment
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
+
+# we propose to refind the same arguments used in first version of GeoXp
+long<-coordinates(sp.obj)[,1]
+lat<-coordinates(sp.obj)[,2]
+
+var1<-sp.obj@data[,names.var[1]]
+var2<-sp.obj@data[,names.var[2]]
+
+listvar<-sp.obj@data
+listnomvar<-names.attr
+
+ # for colors in map and new grahics
+ ifelse(length(col)==1, col2<-"blue", col2<-col)
+ col3<-"lightblue3"
+ 
+# Code which was necessary in the previous version
+ if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
+
+
+ # for identifyng the selected sites
+ifelse(identify, label<-row.names(listvar),label<-"")
 # initialisation
   nointer <- FALSE
   nocart <- FALSE
@@ -19,7 +61,6 @@ if(names.arg[1]=="") names.arg<-levels(as.factor(var))
   lat = as.matrix(lat)
   long = as.matrix(long)
   obs <- vector(mode = "logical", length = length(long))
-  graphics.off()
   graphChoice <- ""
   varChoice1 <- ""
   varChoice2 <- ""
@@ -28,10 +69,9 @@ if(names.arg[1]=="") names.arg<-levels(as.factor(var))
     
   if((length(listvar) > 0)&&(dim(as.matrix(listvar))[2] == 1)) listvar <- as.matrix(listvar)
 
-# ouverture des fentres graphiques    
-  dev.new()
-  dev.new()    
-  fin <- tclVar(FALSE)
+# Windows device
+if(!(2%in%dev.list())) dev.new()
+if(!(3%in%dev.list())) dev.new()
     
 ####################################################
 # slection d'une partie du boxplot
@@ -40,6 +80,11 @@ if(names.arg[1]=="") names.arg<-levels(as.factor(var))
 boxfunc <- function() 
  {
   quit <- FALSE
+ 
+  dev.set(3)
+  title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+  title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
   while (!quit) 
    {
     dev.set(3)
@@ -47,6 +92,8 @@ boxfunc <- function()
     if (is.null(loc)) 
     {
      quit <- TRUE
+     graphique(var1 = var2, var2 = var1, obs = obs, num = 3, graph = "Polyboxplot",
+     labvar = labvar, symbol = pch, couleurs = col, labmod = names.arg,bin=varwidth)
      next
     }
     
@@ -55,7 +102,9 @@ boxfunc <- function()
    
     graphique(var1 = var2, var2 = var1, obs = obs, num = 3, graph = "Polyboxplot",
     labvar = labvar, symbol = pch, couleurs = col, labmod = names.arg,bin=varwidth)
-   
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
     carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,
     lablong=lablong, lablat=lablat, label=label, symbol=pch,carte=carte,nocart=nocart,method="Cluster",
     classe=var1,couleurs=col,legmap=legmap,legends=legends,labmod=names.arg,axis=axes,cex.lab=cex.lab)
@@ -76,7 +125,6 @@ graphfunc <- function()
 {
     if ((length(listvar) != 0) && (length(listnomvar) != 0))
     {
-        dev.off(4)
         choix <<- selectgraph(listnomvar,listgraph)
         varChoice1 <<- choix$varChoice1
         varChoice2 <<- choix$varChoice2
@@ -84,7 +132,7 @@ graphfunc <- function()
             
         if ((length(graphChoice) != 0) && (length(varChoice1) != 0))
         {
-            dev.new()
+            if(!(4%in%dev.list())) dev.new()
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
             obs=obs, num=4, graph=graphChoice, couleurs=col[1], labvar=c(varChoice1,varChoice2))
         }
@@ -144,10 +192,21 @@ SGfunc<-function()
 # quitter l'application
 ####################################################
 
-quitfunc<-function() 
+quitfunc<-function()
 {
-  tclvalue(fin)<<-TRUE
-  tkdestroy(tt);
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+   # assign("obs", row.names(sp.obj)[obs], envir = .GlobalEnv)
+}
+
+quitfunc2<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    print("Results have been saved in last.select object")
+    assign("last.select", which(obs), envir = .GlobalEnv)
 }
 
 ####################################################
@@ -213,8 +272,10 @@ classe=var1,couleurs=col,legmap=legmap,legends=legends,labmod=names.arg,axis=axe
             tkdestroy(tt1)
             msg <- paste("Click on the map to indicate the location of the upper left corner of the legend box")
             tkmessageBox(message = msg)
+            title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
             dev.set(2)
             loc <- locator(1)
+            loc$name <- names(sp.obj[,names.var[1]])
             legends <<- list(legends[[1]], TRUE, legends[[4]], loc)
             carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,
             lablong=lablong, lablat=lablat, label=label, symbol=pch,carte=carte,nocart=nocart,method="Cluster",
@@ -248,72 +309,69 @@ classe=var1,couleurs=col,legmap=legmap,legends=legends,labmod=names.arg,axis=axe
 ####################################################
 # Cration de la bote de dialogue
 ####################################################        
-           
-    if (interactive()) 
-    {
-     tt <- tktoplevel()
-     labelText1 <- tclVar("Work on the polyboxplot")
-     label1 <- tklabel(tt, text = tclvalue(labelText1))
-     tkconfigure(label1, textvariable = labelText1)
-     tkgrid(label1, columnspan = 2)
-     box.but <- tkbutton(tt, text = "Polyboxplot", command = boxfunc)
-     tkgrid(box.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-     
-     label1 <- tclVar("To stop selection, let the cursor on the active graph, click on the right button of the mouse and stop")
-     label11 <- tklabel(tt, justify = "center", wraplength = "3i",text = tclvalue(label1))
-     tkconfigure(label11, textvariable = label1)
-     tkgrid(label11, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-     
-     labelText7 <- tclVar("Preselected sites")
-     label7 <- tklabel(tt, justify = "center", wraplength = "3i",text = tclvalue(labelText7))
-     tkconfigure(label7, textvariable = labelText7)
-     tkgrid(label7, columnspan = 2)
-     noint1.but <- tkbutton(tt, text = "  On/Off  ", command = fnointer)
-     tkgrid(noint1.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-    
-     labelText6 <- tclVar("Draw spatial contours")
-     label6 <- tklabel(tt, justify = "center", wraplength = "3i", text = tclvalue(labelText6))
-     tkconfigure(label6, textvariable = labelText6)
-     tkgrid(label6, columnspan = 2)
-     nocou1.but <- tkbutton(tt, text = "  On/Off  ", command = cartfunc)
-     tkgrid(nocou1.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-     labelText3 <- tclVar("Restore graph")
-     
-     label3 <- tklabel(tt, justify = "center", wraplength = "3i", text = tclvalue(labelText3))
-     tkconfigure(label3, textvariable = labelText3)
-     tkgrid(label3, columnspan = 2)
-     nettoy.but <- tkbutton(tt, text = "     OK     ", command = SGfunc)
-     tkgrid(nettoy.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-     
-     labelText9 <- tclVar("Bubbles")
-     label9 <- tklabel(tt, justify = "center", wraplength = "3i", text = tclvalue(labelText9))
-     tkconfigure(label9, textvariable = labelText9)
-     tkgrid(label9, columnspan = 2)
-     bubble.but <- tkbutton(tt, text = "  On/Off  ", command = fbubble)
-     tkgrid(bubble.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-    
-     labelText4 <- tclVar("Additional graph")
-     label4 <- tklabel(tt, justify = "center", wraplength = "3i", text = tclvalue(labelText4))
-     tkconfigure(label4, textvariable = labelText4)
-     tkgrid(label4, columnspan = 2)
-     autre.but <- tkbutton(tt, text = "     OK     ", command = graphfunc)
-     tkgrid(autre.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-   
-     labelText5 <- tclVar("Exit")
-     label5 <- tklabel(tt, justify = "center", wraplength = "3i", text = tclvalue(labelText5))
-     tkconfigure(label5, textvariable = labelText5)
-     tkgrid(label5, columnspan = 2)
-     quit.but <- tkbutton(tt, text = "     OK     ", command = quitfunc)
-     tkgrid(quit.but, columnspan = 2)
-     tkgrid(tklabel(tt, text = "    "))
-     tkwait.variable(fin)
-    }
-    return(obs)
+ if(interactive())
+{
+fontheading<-tkfont.create(family="times",size=14,weight="bold")
+
+tt <- tktoplevel()
+tkwm.title(tt, "Polyboxplotmap")
+
+frame1a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1a, text = "Interactive selection", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame1a, text = "Work on the graph", font = "Times 12",
+foreground = "darkred", background = "white"))
+point.but <- tkbutton(frame1a, text="Polyboxplotmap", command=boxfunc);
+tkpack(point.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1a, expand = "TRUE", fill = "x")
+
+
+frame1b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nettoy.but <- tkbutton(frame1b, text="     Reset selection     " , command=SGfunc);
+tkpack(nettoy.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1b, expand = "TRUE", fill = "x")
+
+
+frame2 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2, text = "Options", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame2, text = "Spatial contours", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Preselected sites", font = "Times 11",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2, expand = "TRUE", fill = "x")
+
+frame2b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nocou1.but <- tkbutton(frame2b, text="On/Off", command=cartfunc)
+noint1.but <- tkbutton(frame2b, text="On/Off", command=fnointer)
+tkpack(nocou1.but,noint1.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2b, expand = "TRUE", fill = "x")
+
+frame2c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2c, text = "Bubbles", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2c, text = "Additional graph", font = "Times 11",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2c, expand = "TRUE", fill = "x")
+
+frame2d <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+bubble.but <- tkbutton(frame2d, text="On/Off", command=fbubble)
+autre.but <- tkbutton(frame2d, text="     OK     " , command=graphfunc)
+tkpack(bubble.but,autre.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2d, expand = "TRUE", fill = "x")
+
+
+frame3 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame3, text = "Exit", font = "Times 14",
+foreground = "blue", background = "white"))
+
+quit.but <- tkbutton(frame3, text="Save results", command=quitfunc2);
+quit.but2 <- tkbutton(frame3, text="Exit without saving", command=quitfunc);
+
+tkpack(quit.but, quit.but2, side = "left", expand = "TRUE",
+        fill = "x")
+
+tkpack(frame3, expand = "TRUE", fill = "x")
+}
+
+####################################################
+return(invisible())
 }
